package Sentinel::Host;

use strict;
use vars qw( @ISA %AUTOFIELDS $kid );

use Sentinel::Base;
@ISA = qw( Sentinel::Base );

use Sentinel::Test;
use POSIX;

use Data::Dumper;

%AUTOFIELDS = (
	tests => undef,
	addr => undef,
	name => undef,
	sentinel => undef
);

sub OBJ_NEW
{
	my $self = shift;
	$self -> tests( [] );
	return( $self );
}

sub run_tests
{
	my $self = shift;

	$self -> logmsg( "Running tests for: " . $self -> name() );

	for( @{ $self -> tests() } )
	{
		# We fork, the test has a maximum time to complete
		$self -> run_test( $_ );
	}
}

sub run_test
{
	my $self = shift;
	my $test = shift;
	my( $module, $ret, $sig );

	$kid = fork();
	$SIG{ ALRM } = \&timed_out;

	if( ! $kid )
	{
		exit( $test -> run_test( $self ) ); # The exit code is now RET
	}else
	{
		$self -> logmsg( "Setting alarm" );
		alarm( 30 ); # TODO: Make me configurable

		waitpid( $kid, 0 );

		# Grab ret
		$ret = $? >> 8;
		$sig = $? & 127;

		# Disable alarm
		$self -> logmsg( "Disabling alarm" );
		alarm( 0 );
	}

	$self -> logmsg( "Return code is: $ret, signal is: $sig" );
	if( $sig )
	{
		$self -> logmsg( "Test failed, process died of signal $sig" );
		$self -> sentinel() ->
			send_notification( $self -> display_name(),
						"FAIL(" . $_ -> test() . ")",
						"Died of signal: $sig, possible timeout" );
	}elsif( $ret )
	{
		$self -> logmsg( "Test failed" );
		
		# Execute block if onfail is defined
		if( $test -> opt( 'onfail' ) )
		{
			$self -> logmsg( "Executing failure block: " . $test -> opt( 'onfail' ) );
			$self -> execute_block( $test -> opt( 'onfail' ) );
		}
	}else
	{
		$self -> logmsg( "Test passed" );

		# Execute block if onfail is defined
		if( $test -> opt( 'onsuccess' ) )
		{
			$self -> logmsg( "Executing success block: " . $test -> opt( 'onsuccess' ) );
			$self -> execute_block( $test -> opt( 'onsuccess' ) );
		}
	}
}

sub add_test
{
	my $self = shift;
	my $tag = shift;
	my %args = @_;

	push( @{ $self -> tests() }, Sentinel::Test -> new() );

	$self -> last_test() -> test( $tag );
	$self -> last_test() -> args( \%args );
	$self -> last_test() -> host( $self );
}

sub last_test
{
	return( shift() -> tests() -> [ -1 ] );
}

sub display_name
{
	my $self = shift;
	return( $self -> name() || $self -> addr() );
}

sub lib_host
{
	return( shift() -> sentinel() -> lib_host() );
}

sub lib_port
{
	return( shift() -> sentinel() -> lib_port() );
}

sub timed_out
{
	# Happens on timeout on test, kill the child, will call the reaper and set ret
	kill 9, $kid;
}

sub execute_block
{
	my $self = shift;
	my $blockname = shift;

	my $block = $self -> sentinel() -> named_block( $blockname );
	my $test;

	if( ! $block )
	{
		$self -> logmsg( "Undefined block: $blockname" );
		return;
	}

	## WARNING: IF TEST HANDLING CHANGES THIS WILL BREAK!!  UNIFY ME!
	for( @{ $block } )
	{
		$test = Sentinel::Test -> new();
		$test -> test( $_ -> { tag } );
		$test -> args( $_ -> { attr } );
		$test -> cdata( $_ -> { cdata } );
		$test -> host( $self );

		$self -> run_test( $test );
	}
}

1;
